import com.atlassian.jira.component.ComponentAccessor

filename = "${title} - ${new Date().format('yyyy-MM-dd')}.xlsx".toString() // "filename" is a standard Java String, not a GString

transitionHelper = new TransitionHelperTool(user: user)

/**
 * Velocity tool to help with workflow analysis.
 */
public class TransitionHelperTool {
	def user

	/**
	 * @return the list of maps (keyed with "name", "start", "end", "running") that represent the time metrics under SLA.
	 */
	public getTimeMetricsByIssue(def issue){
		def metrics = []

		// get "raw transition" time metrics
		def start = issue.getCreated()

		def changeHistoryManager = ComponentAccessor.changeHistoryManager;
		def changeHistories = changeHistoryManager.getChangeHistoriesForUser(issue, user)
		for(changeHistory in changeHistories) {
			for(changeItem in changeHistory.changeItems) {
				if("status".equals(changeItem.field)) {
					def m = [ name: "Time in \"${changeItem.oldstring}\"", start: start, end: changeHistory.timePerformed, "running": false ]
					metrics.add(m)

					start = changeHistory.timePerformed
				}
			}
		}

		// insert calculated time metrics
		def now = new Date()

		def firstResponseTime = getIssueFirstResponseTime(issue)
		metrics.add([ name: "Time to first response", start: issue.getCreated(), end: firstResponseTime ? firstResponseTime : now, "running": (firstResponseTime == null) ])

		def resolutionTime = getIssueResolutionTime(issue)
		metrics.add([ name: "Time to resolution", start: issue.getCreated(), end: resolutionTime ? resolutionTime : now, "running": (resolutionTime == null) ])

		return metrics
	}

	private getIssueFirstResponseTime(issue) {
		def comments = ComponentAccessor.commentManager.getCommentsForUser(issue, user)

		return !comments.isEmpty() ? comments[0].created : null
	}

	private getIssueResolutionTime(issue) {
		issue.resolutionDate // returns NULL for unresolved
	}
}
